package com.smppw.analysis.application.service.performance;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.map.MapUtil;
import com.smppw.analysis.application.dto.performance.HeadIndicatorParams;
import com.smppw.analysis.application.dto.performance.IndicatorParams;
import com.smppw.common.pojo.enums.Indicator;
import com.smppw.common.pojo.enums.TimeRange;
import com.smppw.constants.Consts;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Service
public class CommonService {
    private final PerformanceService performanceService;

    public CommonService(PerformanceService performanceService) {
        this.performanceService = performanceService;
    }

    public Map<String, Object> getIndicator(HeadIndicatorParams params) {
        Map<String, Object> res = MapUtil.newHashMap();
        // 构建参数体
        IndicatorParams param = prepareParams(params);
        List<Indicator> indicators = ListUtil.of(Indicator.AnnualReturn, Indicator.AnnualStdDev, Indicator.MaxDrawdown, Indicator.SharpeRatio);
        List<Indicator> geoIndicators = ListUtil.of(Indicator.IntervalReturn);
        Map<String, Object> dataset = performanceService.calcIndicators(param, indicators, geoIndicators);
        Map<String, Object> indicatorMap = MapUtil.get(dataset, params.getSecId(), Map.class);
        res.put(Indicator.AnnualReturn.name(), indicatorMap.get(Indicator.AnnualReturn.name()));
        res.put(Indicator.AnnualStdDev.name(), indicatorMap.get(Indicator.AnnualStdDev.name()));
        res.put(Indicator.MaxDrawdown.name(), indicatorMap.get(Indicator.MaxDrawdown.name()));
        res.put(Indicator.SharpeRatio.name(), indicatorMap.get(Indicator.SharpeRatio.name()));
        return res;
    }

    private IndicatorParams prepareParams(HeadIndicatorParams params) {
        IndicatorParams rateRiskIndicatorParams = new IndicatorParams();
        rateRiskIndicatorParams.setSecIds(CollectionUtil.newArrayList());
        rateRiskIndicatorParams.setRefIds(CollectionUtil.newArrayList(params.getSecId()));
        rateRiskIndicatorParams.setBenchmarkId(Consts.BENCHMARK);
        rateRiskIndicatorParams.setNavType(params.getNavType());
        rateRiskIndicatorParams.setRaiseType(params.getRaiseType());
        rateRiskIndicatorParams.setStrategy(params.getStrategy().getStrategyOriginName());
        rateRiskIndicatorParams.setCalcExtraRet(false);
        rateRiskIndicatorParams.setRiskOfFreeId(Consts.RISK_OF_FREE);
        rateRiskIndicatorParams.setTimeRange(TimeRange.FromSetup);
        rateRiskIndicatorParams.setFrequency(params.getFrequency());
        rateRiskIndicatorParams.setStartDate(Consts.START_DATE);
        rateRiskIndicatorParams.setEndDate(DateUtil.formatDate(new Date()));
        return rateRiskIndicatorParams;
    }
}
