package com.smppw.analysis.domain.dao;

import com.smppw.analysis.domain.entity.FundFrequencyDo;
import com.smppw.analysis.domain.entity.FundInformationDo;
import com.smppw.analysis.infrastructure.persistence.FundFrequencyDoMapper;
import com.smppw.analysis.infrastructure.persistence.FundInformationDoMapper;
import com.smppw.common.pojo.enums.Frequency;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Component
public class FundInformationDao {
    private final FundFrequencyDoMapper fundFrequencyDoMapper;
    private final FundInformationDoMapper fundInformationDoMapper;

    public FundInformationDao(FundFrequencyDoMapper fundFrequencyDoMapper, FundInformationDoMapper fundInformationDoMapper) {
        this.fundFrequencyDoMapper = fundFrequencyDoMapper;
        this.fundInformationDoMapper = fundInformationDoMapper;
    }

    public Map<String, Frequency> getNavFrequency(List<String> secIdList) {
        Map<String, Frequency> secFrequencyMap = new HashMap<>();
        List<FundFrequencyDo> fundFrequencyDos = fundFrequencyDoMapper.getNavFrequencyByFundIdList(secIdList);
        for (FundFrequencyDo fundFrequencyDo : fundFrequencyDos) {
            String frequencyStr = fundFrequencyDo.getNavFrequency();
            Frequency frequency = Frequency.Monthly;
            if ("天".equals(frequencyStr)) {
                frequency = Frequency.Daily;
            } else if ("周".equals(frequencyStr)) {
                frequency = Frequency.Weekly;
            }
            secFrequencyMap.put(fundFrequencyDo.getFundId(), frequency);
        }
        return secFrequencyMap;
    }

    public Map<String, String> getMarketFundIdNameMap(List<String> fundIdList) {
        List<FundInformationDo> fundInformationDos = fundInformationDoMapper.listFundInfo(fundIdList);
        Map<String, String> fundIdNameMap = new HashMap<>();
        for (FundInformationDo fundInformationDo : fundInformationDos) {
            fundIdNameMap.put(fundInformationDo.getFundId(), fundInformationDo.getFundShortName());
        }
        return fundIdNameMap;
    }
}
